#! /bin/bash
# ----------------------------------------------------------------------------
# releasePackage_perform.sh - Perform release build after preparation steps.
#
# Environment variables must be set for proper functionality!
#
# Creation   :  2013-11-13  starwarsfan
#
# Copyright (c) 2013-2018 the eisfair team, team(at)eisfair(dot)org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> /tmp/releasePackage-trace$$.txt
#set -x

# Backup where we came from
callDir=$(pwd)
ownLocation="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
scriptName=$(basename $0)
cd ${ownLocation}
. include/helpers_console.sh
rtc=0

helpMe ()
{
    echo "

    Usage:
    ${0##*/} [options]
        This script releases the package which is given by the environment
        variable PACKAGE_TO_RELEASE or the corresponding cmdline option.
        Additionally the env var JOB_NAME must be set (or given via cmdline
        option) to determine base Alpine release and architecture. See
        description below. If both not given, the script will fail.
        Script should be executed out of repository root.

    Optional parameters:
    -b <branch>
        The branch to be used on the repository. Default value: '${BRANCH}'
    -j <job-name>
        Set variable JOB_NAME. Value must be set using the following schema:
        '<some-text>__<alpine-release>_<architecture>' and corresponds to the
        name of the Jenkins job, e. g.:
        - releasePackage__v2.7_x86
        - releasePackage__v3.1_x86_64
    -k <amount>
        Amount of versions per package to keep in place,
        all other (old) versions will be deleted. Default value: ${PKG_AMOUNT_TO_KEEP}
    -p <package-name>
        The package which should be released.
    -r <path>
        Folder, where the released package should be placed. Default: ${CI_RESULTFOLDER_EISFAIR_NG}
    -h  Show this help.
    "
}



checkEnvironment ()
{
    info "##### Checking environment:"
    if [ -z "${JOB_NAME}" ] ; then
        helpMe
        die 10 "Env var 'JOB_NAME' must be set!"
    fi
    if [ -z "${PACKAGE_TO_RELEASE}" ] ; then
        helpMe
        die 11 "Env var 'PACKAGE_TO_RELEASE' must be set!"
    fi
    if [ -z "${EISFAIR_NG_EDGE}" ] ; then
        helpMe
        die 12 "Env var 'EISFAIR_NG_EDGE' must be set!"
    fi
    if [ -z "${EISFAIR_NG_STABLE}" ] ; then
        helpMe
        die 13 "Env var 'EISFAIR_NG_STABLE' must be set!"
    fi
    if [ -z "${EISFAIR_NG_PREVIOUS}" ] ; then
        helpMe
        die 14 "Env var 'EISFAIR_NG_PREVIOUS' must be set!"
    fi
    if [ -z "${CI_RESULTFOLDER_EISFAIR_NG}" ] ; then
        helpMe
        die 15 "Env var 'CI_RESULTFOLDER_EISFAIR_NG' must be set!"
    fi
    if [ ! -e "../${PACKAGE_TO_RELEASE}/APKBUILD" ] ; then
        die 16 "${PACKAGE_TO_RELEASE}/APKBUILD not found, nothing to release!"
    fi
    echo "Done"
}



extractVariables ()
{
    # Extract package name from <some-text>__<alpine-release>_<arch>
    # Example:
    # eisfair-ng/releasePackage__edge_x86
    # eisfair-ng/releasePackage__edge_x86_64
    # eisfair-ng/releasePackage__stable_x86
    # eisfair-ng/releasePackage__stable_x86_64
    # eisfair-ng/releasePackage__previous_x86
    # eisfair-ng/releasePackage__previous_x86_64
    alpineRelease=$(echo ${JOB_NAME} | sed "s/\(.*__\)\([^_]*\)\(_\)\(.*\)/\2/g")
    packageArch=$(echo ${JOB_NAME}   | sed "s/\(.*__\)\([^_]*\)\(_\)\(.*\)/\4/g")
    # If stable or previous, replace with current corresponding version like v2.7 oder v2.6
    if [ "$alpineRelease" = 'stable' ] ; then
        alpineRelease=${EISFAIR_NG_STABLE}
    elif [ "$alpineRelease" = 'previous' ] ; then
        alpineRelease=${EISFAIR_NG_PREVIOUS}
    fi
    info "Building for Alpine Linux release '$alpineRelease' and architecture '$packageArch'"
}



checkDeploymentDestination ()
{
    if [ ! -d ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${BRANCH}/${packageArch} ] ; then
        info "Repository directory not existing, creating it"
        mkdir -p ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${BRANCH}/${packageArch} || die 30 "Unable to create release result directory"
    fi
}



mergeToMaster ()
{
    executeCommand "git reset --hard HEAD" 40
    executeCommand "git checkout ${BRANCH}" 41
    executeCommand "git pull" 42
    executeCommand "git checkout master" 43
    executeCommand "git pull" 44
    executeCommand "git merge origin/${BRANCH}" 45
}



setupReleaseVersionEntries ()
{
    if [ -z "${PKG_VER}" ] ; then
        releasePkgVersion=$(grep "^pkgver" APKBUILD | cut -d "=" -f2)
        if [ -z "${releasePkgVersion}" ] ; then
            die 50 "Error extracting current package version"
        fi
    else
        releasePkgVersion=${PKG_VER}
    fi
    if [ -z "${PKG_REL}" ] ; then
        releasePkgRel=$(grep "^pkgrel" APKBUILD | cut -d "=" -f2)
        if [ -z "${releasePkgRel}" ] ; then
            die 51 "Error extracting current package rel"
        fi
    else
        releasePkgRel=${PKG_REL}
    fi
    nextPkgRel=$((releasePkgRel+1))
    info "Releasing with pkgver ${releasePkgVersion} and pkgrel ${releasePkgRel}"
}



updateVersionEntries ()
{
    info "Updating version entries on APKBUILD"
    sed -i -e "s/^pkgver=.*/pkgver=${releasePkgVersion}/g" \
           -e "s/^pkgrel=.*/pkgrel=${releasePkgRel}/g" \
           APKBUILD
    if git diff-index --quiet HEAD APKBUILD ; then
        info "No changes on ${PACKAGE_TO_RELEASE}/APKBUILD"
    else
        info "${PACKAGE_TO_RELEASE}/APKBUILD updated, committing changes"
        executeCommand "git add APKBUILD" 60
        executeCommand "git commit -m '[Release-Build]: Updated ${PACKAGE_TO_RELEASE}/APKBUILD with release version ${releasePkgVersion}-${releasePkgRel}'" 61
    fi
    executeCommand "git tag '${PACKAGE_TO_RELEASE}-${releasePkgVersion}-${releasePkgRel}'" 62
}



releasePackage ()
{
    info "Updating pkg repository"
    sudo apk update

    info "Removing previously build apk files"
    rm -f *.apk

#    echo "Updating checksums"
#    abuild checksum

    info "Building package"
    abuild -r
    rtc=$?
    if [ "${rtc}" = 0 ] ; then
        info "Copying apk file(s) to repository folder"
        if ls *.apk >/dev/null 2>&1 ; then
            cp -f *.apk ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${BRANCH}/${packageArch}
            rtc=$?
        else
            cp -f ~/packages/${PACKAGE_TO_RELEASE}/${packageArch}/*.apk ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${BRANCH}/${packageArch}
            rtc=$?
        fi
        if [ "${rtc}" != 0 ] ; then
            die ${rtc} " - Unable to copy package to destination folder!"
        fi
    else
        die ${rtc} " - Package release build failed!"
    fi
}



cleanupPackageFolder ()
{
    info "Removing old packages"
    ./cleanupPackageFolder.sh -v ${alpineRelease} -b ${BRANCH} -a ${packageArch} -s ${PKG_AMOUNT_TO_KEEP}
    rtc=$?
    if [ ${rtc} != 0 ] ; then
        warning " - Old packages could not be removed!"
    fi
}



updateRepoIndex ()
{
    # Example: -v v2.7 -b develop -a x86_64
    info "Calling './createRepoIndex.sh -v ${alpineRelease} -b ${BRANCH} -a ${packageArch}'"
    ./createRepoIndex.sh -v ${alpineRelease} -b ${BRANCH} -a ${packageArch}
    rtc=$?
    if [ ${rtc} != 0 ] ; then
        die ${rtc} " - Repository index could not be updated!"
    fi
}

BRANCH='develop'
JOB_NAME=''
PKG_AMOUNT_TO_KEEP=3
PACKAGE_TO_RELEASE=''
CI_RESULTFOLDER_EISFAIR_NG=/tmp/e-ng-releases/
PKG_VER=''
PKG_REL=''

while getopts b:j:p:r:h? option; do
    case ${option} in
        b) BRANCH="${OPTARG}";;
        j) JOB_NAME="${OPTARG}";;
        k) PKG_AMOUNT_TO_KEEP="${OPTARG}";;
        p) PACKAGE_TO_RELEASE="${OPTARG}";;
        r) CI_RESULTFOLDER_EISFAIR_NG="${OPTARG}";;
        h|?) helpMe && exit 0;;
        *) die 90 "invalid option \"${OPTARG}\"";;
    esac
done

checkEnvironment
extractVariables
checkDeploymentDestination
mergeToMaster

info "Cd to ${PACKAGE_TO_RELEASE}"
cd ../${PACKAGE_TO_RELEASE}

setupReleaseVersionEntries
updateVersionEntries
releasePackage
cleanupPackageFolder
updateRepoIndex

cd - > /dev/null

exit ${rtc}

# ============================================================================
# End
# ============================================================================
