#! /bin/bash
# ----------------------------------------------------------------------------
# releasePackage_prepare.sh - Do all neccessary steps to prepeare the release
# of a package.
#
# Environment variables must be set for proper functionality!
#
# Creation   :  2013-11-13  starwarsfan
#
# Copyright (c) 2013-2018 the eisfair team, team(at)eisfair(dot)org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> /tmp/releasePackage-trace$$.txt
#set -x

# Backup where we came from
callDir=$(pwd)
ownLocation="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
scriptName=$(basename $0)
cd ${ownLocation}
. include/helpers_console.sh
rtc=0

helpMe ()
{
    echo "

    Usage:
    ${0##*/} [options]
        This script performs all neccessary preparation steps for the release
        build of the given package.
        The env var JOB_NAME must be set (or given via cmdline option) to
        determine base Alpine release and architecture. See description below.

    Options:
    -b <branch>
        The branch to be used on the repository. Default value: '${BRANCH}'
    -j <job-name>
        Set variable JOB_NAME. Value must be set using the following schema:
        '<some-text>__<alpine-release>_<architecture>' and corresponds to the
        name of the Jenkins job, e. g.:
        - releasePackage__v2.7_x86
        - releasePackage__v3.1_x86_64
    -p <package-name>
        The package which should be released.
    -r <pkg-release>
        Pkg release value to use for release. Value of pkgrel on package APKBUILD
        will be updated.
    -v <pkg-version>
        Pkg version value to use for release. Value of pkgver on package APKBUILD
        will be updated.
    -h  Show this help.
    "
}



checkEnvironment ()
{
    info "##### Checking environment:"
    if [ -z "${JOB_NAME}" ] ; then
        helpMe
        die 10 "Env var 'JOB_NAME' must be set!"
    fi
    if [ -z "${PACKAGE_TO_RELEASE}" ] ; then
        helpMe
        die 11 "Env var 'PACKAGE_TO_RELEASE' must be set!"
    fi
    if [ -z "${EISFAIR_NG_EDGE}" ] ; then
        helpMe
        die 12 "Env var 'EISFAIR_NG_EDGE' must be set!"
    fi
    if [ -z "${EISFAIR_NG_STABLE}" ] ; then
        helpMe
        die 13 "Env var 'EISFAIR_NG_STABLE' must be set!"
    fi
    if [ -z "${EISFAIR_NG_PREVIOUS}" ] ; then
        helpMe
        die 14 "Env var 'EISFAIR_NG_PREVIOUS' must be set!"
    fi
    if [ ! -e "../${PACKAGE_TO_RELEASE}/APKBUILD" ] ; then
        die 16 "${PACKAGE_TO_RELEASE}/APKBUILD not found, nothing to release!"
    fi
    echo "Done"
}



mergeToMaster ()
{
    executeCommand "git reset --hard HEAD" 40
    executeCommand "git checkout ${BRANCH}" 41
    executeCommand "git pull" 42
    executeCommand "git checkout master" 43
    executeCommand "git pull" 44
    executeCommand "git merge origin/${BRANCH}" 45
}



setupReleaseVersionEntries ()
{
    if [ -z "${PKG_VER}" ] ; then
        releasePkgVersion=$(grep "^pkgver" APKBUILD | cut -d "=" -f2)
        if [ -z "${releasePkgVersion}" ] ; then
            die 50 "Error extracting current package version"
        fi
    else
        releasePkgVersion=${PKG_VER}
    fi
    if [ -z "${PKG_REL}" ] ; then
        releasePkgRel=$(grep "^pkgrel" APKBUILD | cut -d "=" -f2)
        if [ -z "${releasePkgRel}" ] ; then
            die 51 "Error extracting current package rel"
        fi
    else
        releasePkgRel=${PKG_REL}
    fi
    nextPkgRel=$((releasePkgRel+1))
    info "Releasing with pkgver ${releasePkgVersion} and pkgrel ${releasePkgRel}"
}



prepareReleaseVersionEntries ()
{
    info "Updating version entries on APKBUILD"
    sed -i -e "s/^pkgver=.*/pkgver=${releasePkgVersion}/g" \
           -e "s/^pkgrel=.*/pkgrel=${releasePkgRel}/g" \
           APKBUILD
    if git diff-index --quiet HEAD APKBUILD ; then
        info "No changes on ${PACKAGE_TO_RELEASE}/APKBUILD"
    else
        info "${PACKAGE_TO_RELEASE}/APKBUILD updated, committing changes"
        executeCommand "git add APKBUILD" 60
        executeCommand "git commit -m '[Prepare release build]: Updated ${PACKAGE_TO_RELEASE}/APKBUILD with release version ${releasePkgVersion}-${releasePkgRel}'" 61
    fi
    executeCommand "git tag '${PACKAGE_TO_RELEASE}-${releasePkgVersion}-${releasePkgRel}'" 62
}



updateSourceBranch ()
{
    info "Updating source branch"
    executeCommand "git checkout ${BRANCH}" 70
    executeCommand "git merge master" 71
    sed -i -e "s/^pkgrel=.*/pkgrel=${nextPkgRel}/g" \
           APKBUILD
    if git diff-index --quiet HEAD APKBUILD ; then
        info "No changes on ${PACKAGE_TO_RELEASE}/APKBUILD"
    else
        info "${PACKAGE_TO_RELEASE}/APKBUILD updated for next development iteration, committing changes"
        executeCommand "git add APKBUILD" 72
        executeCommand "git commit -m '[Prepare release build]: Updated ${PACKAGE_TO_RELEASE}/APKBUILD with next pkgrel version ${nextPkgRel}'" 73
    fi
}



pushChanges ()
{
    info "Pushing release preparation"
    executeCommand "git push origin --all" 80
    executeCommand "git push origin --tags" 81
}



BRANCH='develop'
JOB_NAME=''
PACKAGE_TO_RELEASE=''
PKG_VER=''
PKG_REL=''

while getopts b:j:p:r:v:h? option; do
    case ${option} in
        b) BRANCH="${OPTARG}";;
        j) JOB_NAME="${OPTARG}";;
        p) PACKAGE_TO_RELEASE="${OPTARG}";;
        r) PKG_REL="${OPTARG}";;
        v) PKG_VER="${OPTARG}";;
        h|?) helpMe && exit 0;;
        *) die 90 "invalid option \"${OPTARG}\"";;
    esac
done

checkEnvironment
mergeToMaster

info "Cd to ${PACKAGE_TO_RELEASE}"
cd ../${PACKAGE_TO_RELEASE}

setupReleaseVersionEntries
prepareReleaseVersionEntries
updateSourceBranch
pushChanges

cd - > /dev/null

exit ${rtc}

# ============================================================================
# End
# ============================================================================
